'use strict';

var path = require('path');
var async = require('async');

var PluginError = require('gulp-util').PluginError;
var colors = require('gulp-util').colors;
var log = require('gulp-util').log;

var qn = require('qn');
var merge = require('merge');

var through = require('through2');

module.exports = function (options) {
    return through.obj(function (file, enc, callback) {
        if (file.isNull()) {
            return callback(null, file);
        }

        options = merge({}, options);

        if (!options.qiniu) {
            return callback(null, file);
        }

        var client = qn.create(options.qiniu);
        var filepath = file.path.split(options.baseDir)[1].replace(/\\/g, '/');
        var fileName = path.basename(file.path);

        return async.auto({
            stat: function (cb) {
                client.stat(filepath, function (err, stat) {
                    if (err) {
                        cb(null, true);
                    } else {
                        file.path = options.qiniu.domain + filepath;
                        log('Skip:', colors.grey(fileName));
                        cb(null, false);
                    }
                });
            },
            upload: ['stat', function (results, cb) {
                if (results.stat) {
                    client.uploadFile(file.path, { key: filepath }, function (err, result) {
                        if (err) {
                            cb(err);
                        } else {
                            log('Upload:', colors.green(result.url));
                            file.path = result.url;
                            cb(null, result);
                        }
                    });
                } else {
                    cb(null);
                }
            }]
        }, function (err) {
            if (err) {
                log('Error', colors.red(new PluginError('gulp-qniu', err).message));
            }
            
            callback(null, file);
        });
    });
};
